% *
% * determineThromboParameters.m
% *
% *      Author: neeb
% 
% Copyright 2013 Heiko Neeb
% 
% This file is part of the Grnwald Remagen OpenSource Unified 
% Platelet-Identification Tracker (GROUP-IT) toolkit.
% 
% GROUP-IT is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% GROUP-IT is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with GROUP-IT.  If not, see <http://www.gnu.org/licenses/>.
% 
% GROUP-IT is designed to be used for scientific purposes only. Do NOT
% base any diagnosis on the results obtained with this program


function [ thromboResults, cnt, curSegment ] = determineThromboParameters( curSegment, scale, removeLeukos,...
    LeukocyteMinGrayVal,scatterGroupSlopeOffset, determinePseudoPods, filename, ...
    filenameExtension, SeriesID)

% allocate 
thromboResults=zeros(1,36);
cnt=0;     
SE=strel('disk',1);
filenameOrig=filename;


% slope and offsets for scatter group (FD-vs. Perim) assignment
s0=scatterGroupSlopeOffset(1,1);
o0=scatterGroupSlopeOffset(1,2);
s1=scatterGroupSlopeOffset(2,1);
o1=scatterGroupSlopeOffset(2,2);
s2=scatterGroupSlopeOffset(3,1);
o2=scatterGroupSlopeOffset(3,2);
    
    
% label segment    
[curImageLabeled, nThrombos]=bwlabel(curSegment);
for(thro=1:nThrombos)
    
    curLabMask=curImageLabeled==thro;
    
    % check if current thrombo contains multiple segments. If so,
    % continue for the moment
    tmpLabel=bwlabel(curLabMask);
    if(max(tmpLabel(:))>1 )
        continue;
    end
    
    cnt=cnt+1;
    
    % Area
    thromboArea=length(curLabMask(curLabMask>0));
    if(thromboArea==0)
        continue;
    end
    
    thromboResults(cnt,10)=thromboArea;
    
    % dilate thrombo
    curLabMask=imdilate(curLabMask,SE);
    
    STATS = regionprops(curLabMask, 'BoundingBox');
    ULCorner=max([1 1], floor(STATS.BoundingBox(1:2)));
    RegExtend=ceil(STATS.BoundingBox(3:4));
    
    curThrombo=curLabMask(ULCorner(2):ULCorner(2)+RegExtend(2),ULCorner(1):ULCorner(1)+RegExtend(1));
    curThrombo=edge(imfill(curThrombo,'holes'),'log');
    
    % Coordinates
    thromboResults(cnt,9)=ceil(1*(ULCorner(1)+RegExtend(1)/2));
    thromboResults(cnt,10)=ceil(1*(ULCorner(2)+RegExtend(2)/2));
    
    % Perimeter
    thromboResults(cnt,12)=length(curThrombo(curThrombo>0))*scale;
    
    % Re-determine area
    curLabMask=imerode(imfill(curLabMask,'holes'),SE);
    thromboArea=length(curLabMask(curLabMask>0));
    thromboResults(cnt,11)=thromboArea*scale^2;
    
    % Cirularity
    thromboResults(cnt,14)=min(5,thromboResults(cnt,11)./(pi*(thromboResults(cnt,12)/2/pi).^2));
    
    
    % Some misc binary region properties offered by Matlab
    s=regionprops(curLabMask, 'Eccentricity', 'EquivDiameter', 'Extent',...
        'MajorAxisLength','MinorAxisLength','Solidity');
    
    thromboResults(cnt,17)=s.Eccentricity;
    thromboResults(cnt,18)=s.EquivDiameter;
    thromboResults(cnt,19)=s.Extent;
    thromboResults(cnt,20)=s.MajorAxisLength;
    thromboResults(cnt,21)=s.MinorAxisLength;
    thromboResults(cnt,22)=s.Solidity;
    
    
    % Average Grey Value of Segment
    thromboResults(cnt,15)=mean(curSegment(curLabMask>0));
    
    
    % Std of Segment Grey Value
    thromboResults(cnt,16)=std(curSegment(curLabMask>0));
    
    
    % get fractal dimension
    c=boxcount(curThrombo);
    boxsize=2.^(0:length(c)-1);
    pval=polyfit(log(boxsize(3:end)),log(c(3:end)),1);
    thromboResults(cnt,13)=-1*pval(1);
    
    % check if Leukocytes should be removed
    if( removeLeukos > 0 )
        if( thromboResults(cnt,13) > 1.25 && ... %FD
                thromboResults(cnt,11) > 90 && ... %Area
                thromboResults(cnt,15) > LeukocyteMinGrayVal && ... %Av GW
                thromboResults(cnt,14) > 0.75 ) % Perimeter
            
            % Remove segment from image
            curSegment(curLabMask>0)=0;
            cnt = cnt-1;
            continue;
        else
            curLabelOrig=curSegment;
            curLabelOrig(curLabMask==0)=0;
            %                    figure, imagesc(curLabelOrig)
            curLabelOrig(curLabelOrig<quantile(curLabelOrig(curLabMask>0),0.5))=0;
            %                    close all
            
            % check if multiple segments are present. If so, take only the
            % largest (presumably a Ery/Leuko
            [curSegmentReabled,nLabelsRelabeled]=bwlabel(curLabelOrig>0);
            if(nLabelsRelabeled > 1)
                relabelHist=histc(curSegmentReabled(curSegmentReabled>0),...
                    1:nLabelsRelabeled);
                maxRelabelId=find(relabelHist==max(relabelHist));
                curLabelOrig(curSegmentReabled~=maxRelabelId(1))=0;
            end
            
            % determine parameters for shrunken area
            shrunkenAreaMedianGW=median(curLabelOrig(curLabelOrig>0));
            shrunkenArea=length(curLabelOrig(curLabelOrig>0))*scale*scale;
            shrunkenGW=mean(curLabelOrig(curLabelOrig>=shrunkenAreaMedianGW));
            shrunkenGWVariance=std(curLabelOrig(curLabelOrig>=...
                shrunkenAreaMedianGW))/shrunkenGW;
            curLabelOrig=edge(curLabelOrig>0);
            shrunkenPeri=length(curLabelOrig(curLabelOrig>0))*scale;
            shrunkenCirc=shrunkenArea/(pi*(shrunkenPeri/2/pi)^2);
            
            if( shrunkenArea > 50 && shrunkenCirc>0.5 && ...
                    ( shrunkenGW>LeukocyteMinGrayVal ...
                    || shrunkenGWVariance<0.1) )
                % Remove segment from image
                curSegment(curLabMask>0)=0;
                cnt = cnt-1;
                continue;
            end
        end
    end
    
    
    % determine scatter group assignment
    x=thromboResults(cnt,12);
    y=thromboResults(cnt,13);
    y0=x*s0+o0;
    y1=x*s1+o1;
    y2=x*s2+o2;
    if( y > y0 )
        % Group 1
        thromboResults(cnt,28)=1;
    else
        if( y <= y0 && y>y1 )
            % Group 2
            thromboResults(cnt,28)=2;
        else
            if( y<y2 )
                % Group 4
                thromboResults(cnt,28)=4;
            else
                % Group 3
                thromboResults(cnt,28)=3;
            end                
        end
    end
    
    
    % check if pseuodpod parameters should be determined
    if( determinePseudoPods > 0 )
        [TA,pseudDiameter,pseudDist,pseudArea,nPseudoVec,bodyDiameterVec]=...
            PseudopodDetection(curLabMask,0, scale);
        
        if( length(nPseudoVec)> 0 )
            tmpPseudoArea=pseudArea(1,:);
            tmpPseudoDiameter=pseudDiameter(1,:);
            tmpPseudoDist=pseudDist(1,:);
            thromboResults(cnt,23)=nPseudoVec(1);
            thromboResults(cnt,24)=bodyDiameterVec(1);
            thromboResults(cnt,25)=mean(tmpPseudoArea(tmpPseudoArea>0));
            thromboResults(cnt,26)=mean(tmpPseudoDiameter(tmpPseudoArea>0));
            thromboResults(cnt,27)=mean(tmpPseudoDist(tmpPseudoArea>0));
            
            %ImagesWithPseudopods(:,:,i)=ImagesWithPseudopods(:,:,i)+TA;
        end
    end
    
    
    % fill in file info if available
    if( length(filenameOrig) > 0 )
        
        filename=filenameOrig;
        filename=strrep(filename,'_',' ');
        filename=strrep(filename,filenameExtension,'');
        
        [bildID, filename]=strtok(filename); bildID=str2num(bildID);
        [Year, filename]=strtok(filename); Year=str2num(Year);
        [MonthDay, filename]=strtok(filename); Month=str2num(MonthDay(1:2)); Day=str2num(MonthDay(3:4));
        [Hour, filename]=strtok(filename); Hour=str2num(Hour);
        [Minute, filename]=strtok(filename);Minute=str2num(Minute);
        [Second, filename]=strtok(filename);Second=str2num(Second);
        
        thromboResults(cnt,1)=SeriesID;
        thromboResults(cnt,2)=bildID;
        thromboResults(cnt,3)=Year;
        thromboResults(cnt,4)=Month;
        thromboResults(cnt,5)=Day;
        thromboResults(cnt,6)=Hour;
        thromboResults(cnt,7)=Minute;
        thromboResults(cnt,8)=Second;
    end
end


end

